@@ -196,9 +196,11 @@ const char * dummy[] = {
 "",
 "/// XAPIAN_AT_LEAST(A,B,C) checks for xapian-core >= A.B.C - use like so:",
 "///",
+"/// @code",
 "/// #if XAPIAN_AT_LEAST(1,4,2)",
 "/// /* Code needing features needing Xapian >= 1.4.2. */",
 "/// #endif",
+"/// @endcode",
 "///",
 "/// Added in Xapian 1.4.2.",
 "#define XAPIAN_AT_LEAST(A,B,C) \\",
